/*******************************************************************************
*   Filename:       modbus_slave.h
*   Revised:        All copyrights reserved to Roger-WY.
*   Date:           2016-06-29
*   Revision:       v1.0
*   Writer:	        Roger-WY
*
*   Description:     Modbus通信模块.  头文件
*                    从站模式
*******************************************************************************/

#ifndef __MODBUY_SLAVE_H__
#define __MODBUY_SLAVE_H__


#include "modbus_cfg.h"
#include "type_def.h"

#if (MODBUS_CFG_SLAVE_EN == 1)

//-----------------------------------------------------------------------------------------//
#define MODBUS_SLAVE_RTU_01H_FUNCTION ENABLE
#define MODBUS_SLAVE_RTU_02H_FUNCTION ENABLE
#define MODBUS_SLAVE_RTU_03H_FUNCTION ENABLE
#define MODBUS_SLAVE_RTU_04H_FUNCTION ENABLE
#define MODBUS_SLAVE_RTU_05H_FUNCTION ENABLE
#define MODBUS_SLAVE_RTU_06H_FUNCTION ENABLE
#define MODBUS_SLAVE_RTU_10H_FUNCTION ENABLE


#define MODBUS_RTU_TEST ENABLE//DISABLE  ENABLE


//-----------------------------------------------------------------------------------------//
/* RTU 应答代码 */
#define RSP_OK				0		/* 成功 */
#define RSP_ERR_CMD			0x01	/* 不支持的功能码 */
#define RSP_ERR_REG_ADDR	0x02	/* 寄存器地址错误 */
#define RSP_ERR_VALUE		0x03	/* 数据值域错误 */
#define RSP_ERR_WRITE		0x04	/* 写入失败 */

//-----------------------------------------------------------------------------------------//
#if (MODBUS_RTU_TEST == ENABLE)   //测试使用代码
/* 01H 读强制单线圈 */
/* 05H 写强制单线圈 */
#define REG_D01		0x0101   //线圈首地址
#define REG_DXX 	0x0180	 //线圈末地址

/* 02H 读取输入状态 */
#define REG_T01		0x0201
#define REG_T02		0x0202
#define REG_T03		0x0203
#define REG_TXX		REG_T03

/* 03H 读保持寄存器 */
/* 06H 写保持寄存器 */
/* 10H 写多个保存寄存器 */
#define SLAVE_REG_P01		0x0301//寄存器首地址
#define SLAVE_REG_PXX		0x0381//寄存器末地址
/* 04H 读取输入寄存器(模拟信号) */
#define REG_A01		0x0401
#define REG_AXX		REG_A01
typedef struct
{
	/* 01H 05H 读写单个强制线圈 */
	uint16_t D01[128];//数组
	/* 03H 06H 读写保持寄存器 */
	uint16_t P01[128];
	/* 04H 读取模拟量寄存器 */
	uint16_t A01;


}SLAVE_VAR_T;
#endif

//-----------------------------------------------------------------------------------------//
typedef struct
{
	uint8_t RxBuf[S_RX_BUF_SIZE];
	uint8_t RxCount;
	uint8_t RxStatus;
	uint8_t RxNewFlag;
	uint8_t RspCode;
	uint8_t TxBuf[S_TX_BUF_SIZE];
	uint8_t TxCount;
}MODS_T;



//-----------------------------------------------------------------------------------------//
void MODS_Poll(void);
void MODS_PeripheralInit(void);
void MODS_RxTimeOut(void);
void MODS_ReciveNew(uint8_t _byte);
//-----------------------------------------------------------------------------------------//
extern MODS_T      xdata g_tModS;
#if (MODBUS_RTU_TEST == ENABLE)
extern SLAVE_VAR_T xdata g_tVar;
#endif

#endif

#endif

/*****************************  (END OF FILE) *********************************/
